<?php
/**
 * This file implements the Template class.
 *
 * This file is part of Quam Plures - {@link http://quamplures.net/}
 * See also {@link https://launchpad.net/quam-plures}.
 *
 * @copyright (c) 2009 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 * Parts of this file are copyright (c)2005-2006 by PROGIDISTRI - {@link http://progidistri.com/}.
 *
 * {@internal License choice
 * - If you have received this file as part of a package, please find the license.txt file in
 *   the same folder or the closest folder above for complete license terms.
 * - If you have received this file individually (e-g: from http://evocms.cvs.sourceforge.net/)
 *   then you must choose one of the following licenses before using the file:
 *   - GNU General Public License 2 (GPL) - http://www.opensource.org/licenses/gpl-license.php
 *   - Mozilla Public License 1.1 (MPL) - http://www.opensource.org/licenses/mozilla1.1.php
 * }}
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file: }}
 * @author fplanque: Francois PLANQUE.
 *
 * @package templates
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

/**
 * Template Class
 *
 * @package templates
 */
class Template extends DataObject
{
	var $name;
	var $folder;
	var $type;

	/**
	 * Lazy filled.
	 * @var array
	 */
	var $container_list = NULL;

	/**
	 * The translations keyed by locale. They get loaded through include() of _global.php.
	 * @see Template::T_()
	 * @var array
	 */
	var $_trans = array();

	/**
	 * Our own application credits (lazy filled by constructor).
	 *
	 * These are displayed by the "credits" display handler.
	 *
	 * @var array
	 */
	var $poweredby_linkback;


	/**
	 * Constructor
	 *
	 * @param table Database row
	 */
	function Template( $db_row = NULL, $template_folder = NULL )
	{
		// Call parent constructor:
		parent::DataObject( 'T_templates__template', 'template_', 'template_ID' );

		$this->delete_restrictions = array(
				array( 'table'=>'T_blogs', 'fk'=>'blog_template_ID', 'msg'=>T_('%d blogs using this template') ),
			);

		$this->delete_cascades = array(
				array( 'table'=>'T_templates__container', 'fk'=>'sco_template_ID', 'msg'=>T_('%d linked containers') ),
			);

		global $app_name, $app_homepage;
		$this->poweredby_linkback = array(
			'pre' => T_('Powered by'),
			'text' => $app_name,
			'post' => T_('Quam Plures is fully committed to making the most open open-source app available. Got an idea to improve this product? <a href="http://forums.quamplures.net/" title="link to QP forums">Join our forums</a> and help make the world a better place. Ecce potestas casei!'),
			'url' => $app_homepage,
			'title' => T_('A multi-user multi-blog app made by users who care'),
		);

		if( is_null($db_row) )
		{	// We are creating an object here:
			$this->set( 'folder', $template_folder );
			$this->set( 'name', $this->get_default_name() );
			$this->set( 'type', $this->get_default_type() );
		}
		else
		{	// Wa are loading an object:
			$this->ID = $db_row->template_ID;
			$this->name = $db_row->template_name;
			$this->folder = $db_row->template_folder;
			$this->type = $db_row->template_type;
		}

	}


	/**
	 * Install current template to DB
	 *
	 * @return boolean True on success, false on failure.
	 */
	function install()
	{
		// Look for containers in template file:
		if( ! $this->discover_containers() )
		{	// Error messages already added.
			return false;
		}

		// INSERT NEW TEMPLATE INTO DB:
		$this->dbinsert();
		return true;
	}


	/**
	 * Create default widgets for requested blog
	 *
	 * @todo yabs > change this to param array
	 *
	 * @param obj Blog $edited_Blog
	 */
	function CreateDefaultWidgets( & $edited_Blog )
	{
		global $DB, $Messages;
		if( $default_widgets = $this->get_default_widgets( array( 'Blog' => & $edited_Blog ) ) )
		{ // we have widgets
			foreach( $default_widgets as $container => $widgets )
			{
				$count = 0;
				foreach( $widgets as $widget => $settings )
				{
					$settings = array_merge( array(
						'type' => 'widget',
						'params' => array(),
					), $settings );

					$DB->query( 'INSERT INTO T_widget( wi_coll_ID, wi_sco_name, wi_order, wi_type, wi_code, wi_params )
											 VALUES(
											 	'.$edited_Blog->ID.',
											 	'.$DB->quote( $container ).',
											 	'.( ++$count ).',
											 	'.$DB->quote( $settings['type'] ).',
											 	'.$DB->quote( $widget ).',
											 	'.$DB->quote( $DB->escape( serialize( $settings['params'] ) ) ).' )' );

				}
			}
			$Messages->add( T_('Default widgets have been set-up for this blog.'), 'success' );
		}
	}


	/**
	 * Get default widgets for template
	 *
	 * @todo yabs > document params, default_widgets_structure
	 *
	 * @return array widgets for each container
	 */
	function get_default_widgets( $params = array() )
	{
		global $Blog;
		$params = array_merge( array(
			'Blog' => & $Blog,
		), $params );

		$default_widgets = array(
			'Page Top' => array(
				'bloglist' => array(
					'type' => 'widget',
					'params' => array(
						'coll_list_type' => ( $params['Blog']->get( 'in_bloglist' ) ? 'public' : 'owner' ),
					),
				),
			),

			'Header' => array(
				'title' => array(
					'type' => 'widget',
				),
				'tagline' => array(
					'type' => 'widget',
				),
			),

			'Menu' => array(
				'menu_link' => array(
					'type' => 'widget',
					'params' => array(
						'link_type'=>'home',
					),
				),
				'page_list' => array(
					'type' => 'widget',
				),
				'menu_link' => array(
					'type' => 'widget',
					'params' => array(
						'link_type'=>'ownercontact',
					),
				),
				'menu_link' => array(
					'type' => 'widget',
					'params' => array(
						'link_type'=>'login',
					),
				),
			),

			'Sidebar' => array(
				'qp_calendar' => array(
					'type' => 'plugin',
				),
				'title' => array(
					'type' => 'widget',
				),
				'longdesc' => array(
					'type' => 'widget',
				),
				'common_links' => array(
					'type' => 'widget',
				),
				'search_form' => array(
					'type' => 'widget',
				),
				'category_list' => array(
					'type' => 'widget',
				),
				'xml_feeds' => array(
					'type' => 'widget',
				),
			),
		);


		return $default_widgets;

	}

  /**
	 * Get default name for the template.
	 * Note: the admin can customize it.
	 */
	function get_default_name()
	{
		return $this->folder;
	}


  /**
	 * Get default type for the template.
	 */
	function get_default_type()
	{
		return (substr($this->folder,0,1) == '_' ? 'feed' : 'normal');
	}


	/**
	 * Get the customized name for the template.
	 */
	function get_name()
	{
		return $this->name;
	}


  /**
	 * Get credits for linkback on credits page.
	 *
	 * Each template can and should provide an array for credits for domains/authors
	 * who created the original design, inspired your design, or ported the template
	 * in the following format:
	 * <code>
	 * $template_linkbacks = array(
	 *   'designer' => array(
	 *     'pre' => T_('Design by'),
	 *     'text' => 'linked text',
	 *     'post' => T_('A reasonably brief summary of the template. A sentence or two should be enough?'),
	 *     'url' => 'http://yourdomain.tld/',
	 *     'title' => T_('insert creative link title'),
	 *   ),
	 * );
	 * return $template_linkbacks;
	 * </code>
	 *
	 * Consider "designer" or "inspired" or "ported" as usual and customary types,
	 * and please limit the amount of linkbacks provided in the array. More than
	 * 3 will give a warning to the installer that your template might be spam!
	 *
	 * 'pre' => T_('Design by') or T_('Inspired by') or T_('Ported by')
	 * 'text' => the linked text - your name or domain name for example
	 * 'post' => can be '' (empty), and can hold HTML if you want.  Please keep it reasonably short!
	 * 'url' => REQUIRED (of course).  Without a URL there is no linkback!
	 * 'title' => can be whatever you like, but should never be empty.  Use the URL if nothing else.
	 *
	 * @return array The credits array (see above).
	 */
	function get_template_credits()
	{
		return array();
	}


	/**
	 * Load data from Request form fields.
	 *
	 * @return boolean true if loaded data seems valid.
	 */
	function load_from_Request()
	{
		// Name
		param_string_not_empty( 'template_name', T_('Please enter a name.') );
		$this->set_from_Request( 'name' );

		// Template type
		param( 'template_type', 'string' );
		$this->set_from_Request( 'type' );

		return ! param_errors_detected();
	}


	/**
	 * Load params
	 */
	function load_params_from_Request()
	{
		load_funcs('plugins/_plugin.funcs.php');

		// Loop through all widget params:
		foreach( $this->get_param_definitions( array('for_editing'=>true) ) as $parname => $parmeta )
		{
			autoform_set_param_from_request( $parname, $parmeta, $this, 'Template' );
		}
	}


	/**
	 * Display a container
	 *
	 * @todo (legacy): fp> if it doesn't get any template specific, move it outta here! :P
	 * fp> Do we need Template objects in the frontoffice at all? -- Do we want to include the dispatcher into the Template object? WARNING: globals
	 * fp> We might want to customize the container defaults. -- Per blog or per template?
	 *
	 * @uses ComponentWidget::display()
	 * @uses ComponentWidget::init_display()
	 *
	 * @param string
	 * @param array
	 */
	function container( $sco_name, $params = array() )
	{
		/**
		 * Blog currently displayed
		 * @var Blog
		 */
		global $Blog;
		global $admin_url, $rsc_url;

		$params = array_merge( array( // stop annoying validation errors when container is <ul> and empty
			'container_start' => '<div id="%sco_name%" class="container">',
			'container_end' => '</div>',
		), $params );

		$sco_dom_ID = preg_replace( '~[^a-z0-9]~i', '', ucwords( $sco_name ) );
		$params['container_start'] = str_replace( '%sco_name%', 'sco_'.$sco_dom_ID, $params['container_start'] );

		/**
		 * @var EnabledWidgetCache
		 */
		$EnabledWidgetCache = & get_Cache( 'EnabledWidgetCache' );
		$Widget_array = & $EnabledWidgetCache->get_by_coll_container( $Blog->ID, $sco_name );

		if( !empty($Widget_array) )
		{
			echo $params['container_start']."\n";
			foreach( $Widget_array as $ComponentWidget )
			{
				// Let the Widget display itself (with contextual params):
				$ComponentWidget->display( $params );
			}
			echo $params['container_end']."\n";
		}
	}


	/**
	 * Discover containers included in template file
	 * @todo browse all *.tpl.php
	 */
	function discover_containers()
	{
		global $templates_path, $Messages;

		$this->container_list = array();

		if( ! $dir = @opendir($templates_path.$this->folder) )
		{
			$Messages->add( 'Cannot open template directory.', 'error' ); // No trans
			return false;
		}

		while( ( $file = readdir($dir) ) !== false )
		{
			$rf_main_subpath = $this->folder.'/'.$file;
			$af_main_path = $templates_path.$rf_main_subpath;

			if( !is_file( $af_main_path ) || ! preg_match( '\.php$', $file ) )
			{ // Not a php template file
				continue;
			}

			if( ! is_readable($af_main_path) )
			{
				$Messages->add( sprintf( T_('Cannot read template file &laquo;%s&raquo;!'), $rf_main_subpath ), 'error' );
				continue;
			}

			$file_contents = @file_get_contents( $af_main_path );
			if( ! is_string($file_contents) )
			{
				$Messages->add( sprintf( T_('Cannot read template file &laquo;%s&raquo;!'), $rf_main_subpath ), 'error' );
				continue;
			}


			// if( ! preg_match_all( '~ \$Template->container\( .*? (\' (.+?) \' )|(" (.+?) ") ~xmi', $file_contents, $matches ) )
			if( ! preg_match_all( '~ (\$Template->|template_)container\( .*? ((\' (.+?) \')|(" (.+?) ")) ~xmi', $file_contents, $matches ) )
			{	// No containers in this file
				continue;
			}

			// Merge matches from the two regexp parts (due to regexp "|" )
			$container_list = array_merge( $matches[4], $matches[6] );

			$c = 0;
			foreach( $container_list as $container )
			{
				if( empty($container) )
				{	// regexp empty match
					continue;
				}

				$c++;

				if( in_array( $container, $this->container_list ) )
				{	// we already have that one
					continue;
				}

				$this->container_list[] = $container;
			}

			if( $c )
			{
				$Messages->add( sprintf( T_('%d containers have been found in template template &laquo;%s&raquo;.'), $c, $rf_main_subpath ), 'success' );
			}
		}

		// pre_dump( $this->container_list );

		if( empty($this->container_list) )
		{	// Display a warning:
			$Messages->add( T_('No containers found in this template!'), 'note' );
		}

		return true;
	}

	/**
	 * @return array
	 */
	function get_containers()
	{
    /**
		 * @var DB
		 */
		global $DB;

		if( is_null( $this->container_list ) )
		{
			$this->container_list = $DB->get_col(
				'SELECT sco_name
					 FROM T_templates__container
					WHERE sco_template_ID = '.$this->ID, 0, 'get list of containers for template' );
		}

		return $this->container_list;
	}

	/**
	 * Update the DB based on previously recorded changes
	 *
	 * @return boolean true
	 */
	function dbupdate()
	{
		global $DB;

		$DB->begin();

		if( parent::dbupdate() !== false )
		{	// Template updated, also save containers:
			$this->db_save_containers();
		}

		$DB->commit();

		return true;
	}


	/**
	 * Insert object into DB based on previously recorded changes.
	 *
	 * @return boolean true
	 */
	function dbinsert()
	{
		global $DB;

		$DB->begin();

		if( parent::dbinsert() )
		{	// Template saved, also save containers:
			$this->db_save_containers();
		}

		$DB->commit();

		return true;
	}


	/**
	 * Save containers
	 *
	 * to be called by dbinsert / dbupdate
	 */
	function db_save_containers()
	{
		global $DB;

		if( empty( $this->container_list ) )
		{
			return false;
		}

		$values = array();
		foreach( $this->container_list as $container_name )
		{
			$values [] = '( '.$this->ID.', '.$DB->quote($container_name).' )';
		}

		$DB->query( 'REPLACE INTO T_templates__container( sco_template_ID, sco_name )
									VALUES '.implode( ',', $values ), 'Insert containers' );

		return true;
	}


	/**
	 * Display templateshot for template folder in various places.
	 *
	 * Including for NON installed templates.
	 *
	 * @static
	 */
	function disp_templateshot( $template_folder, $template_name, $function = NULL, $selected = false, $select_url = NULL, $function_url = NULL )
	{
		global $templates_path, $templates_url;

		if( !empty($select_url) )
		{
			$select_a_begin = '<a href="'.$select_url.'" title="'.T_('Select this template!').'">';
			$select_a_end = '</a>';
		}
		else
		{
			$select_a_begin = '<a href="'.$function_url.'" title="'.T_('Install NOW!').'">';
			$select_a_end = '</a>';
		}

		echo '<div class="templateshot">';
		echo '<div class="templateshot_placeholder';
		if( $selected )
		{
			echo ' current';
		}
		echo '">';
		if( file_exists( $templates_path.$template_folder.'/templateshot.png' ) )
		{
			echo $select_a_begin;
			echo '<img src="'.$templates_url.$template_folder.'/templateshot.png" width="240" height="180" alt="'.$template_folder.'" />';
			echo $select_a_end;
		}
		elseif( file_exists( $templates_path.$template_folder.'/templateshot.jpg' ) )
		{
			echo $select_a_begin;
			echo '<img src="'.$templates_url.$template_folder.'/templateshot.jpg" width="240" height="180" alt="'.$template_folder.'" />';
			echo $select_a_end;
		}
		elseif( file_exists( $templates_path.$template_folder.'/templateshot.gif' ) )
		{
			echo $select_a_begin;
			echo '<img src="'.$templates_url.$template_folder.'/templateshot.gif" width="240" height="180" alt="'.$template_folder.'" />';
			echo $select_a_end;
		}
		else
		{
			echo '<div class="templateshot_noshot">'.T_('No templateshot available for').'</div>';
			echo '<div class="templateshot_name">'.$select_a_begin.$template_folder.$select_a_end.'</div>';
		}
		echo '</div>';
		echo '<div class="legend">';
		if( !empty( $function) )
		{
			echo '<div class="actions">';
			switch( $function )
			{
				case 'install':
					echo '<a href="'.$function_url.'" title="'.T_('Install NOW!').'">';
					echo T_('Install NOW!').'</a>';
					break;

				case 'select':
					echo '<a href="'.$function_url.'" target="_blank" title="'.T_('Preview blog with this template in a new window').'">';
					echo T_('Preview').'</a>';
					break;
			}
			echo '</div>';
		}
		echo '<strong>'.$template_name.'</strong>';
		echo '</div>';
		echo '</div>';
	}



	/**
   * Get definitions for editable params
   *
   * @todo this is destined to be overridden by derived Template classes
   *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array();

		return $r;
	}


	/**
 	 * Get a template specific param value from current Blog
 	 *
	 */
	function get_setting( $parname )
	{
		/**
		 * @var Blog
		 */
		global $Blog;

		// Name of the setting in the blog settings:
		$blog_setting_name = 'template'.$this->ID.'_'.$parname;

		$value = $Blog->get_setting( $blog_setting_name );

		if( ! is_null( $value ) )
		{	// We have a value for this param:
			return $value;
		}

		// Try default values:
		$params = $this->get_param_definitions( NULL );
		if( isset( $params[$parname]['defaultvalue'] ) )
		{	// We ahve a default value:
			return $params[$parname]['defaultvalue'] ;
		}

		return NULL;
	}


	/**
	 * Set a template specific param value for current Blog
	 *
	 * @param string parameter name
	 * @param mixed parameter value
	 */
	function set_setting( $parname, $parvalue )
	{
		/**
		 * @var Blog
		 */
		global $Blog;

		// Name of the setting in the blog settings:
		$blog_setting_name = 'template'.$this->ID.'_'.$parname;

		$Blog->set_setting( $blog_setting_name, $parvalue );
	}


	/**
	 * Save template specific settings for current blgo to DB
	 */
	function dbupdate_settings()
	{
		/**
		 * @var Blog
		 */
		global $Blog;

		$Blog->dbupdate();
	}


	/**
	 * Get ready for displaying the template.
	 *
	 * This may register some CSS or JS...
	 */
	function display_init()
	{
		// override in specific templates...
	}


	/**
	 * Translate a given string, in the Template's context.
	 *
	 * This means, that the translation is obtained from the Template's
	 * "locales" folder.
	 *
	 * It uses the global/regular {@link T_()} function as a fallback.
	 *
	 * @param string The string (english), that should be translated
	 * @param string Requested locale ({@link $current_locale} gets used by default)
	 * @return string The translated string.
	 *
	 * @uses T_()
	 * @since 3.2.0 (after beta)
	 */
	function T_( $string, $req_locale = '' )
	{
		global $templates_path;

		if( ( $return = T_( $string, $req_locale, array(
								'ext_transarray' => & $this->_trans,
								'alt_basedir'    => $templates_path.$this->folder,
							) ) ) == $string )
		{	// This template did not provide a translation - fallback to global T_():
			return T_( $string, $req_locale );
		}

		return $return;
	}


	/**
	 * Translate and escape single quotes.
	 *
	 * This is to be used mainly for Javascript strings.
	 *
	 * @param string String to translate
	 * @param string Locale to use
	 * @return string The translated and escaped string.
	 *
	 * @uses Template::T_()
	 * @since 3.2.0 (after beta)
	 */
	function TS_( $string, $req_locale = '' )
	{
		return str_replace( "'", "\\'", $this->T_( $string, $req_locale ) );
	}


}


?>
