<?php
/**
 * This file implements the UI controller for template management.
 *
 * Quam Plures - {@link http://quamplures.net/}
 *
 * @copyright (c) 2009 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 * Parts of this file are copyright (c)2004-2006 by Daniel HAHLER - {@link http://thequod.de/contact}.
 *
 * Released under GNU GPL License - {@link http://quamplures.net/license.html}
 *
 * {@internal Open Source relicensing agreement:
 * }}
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file: }}
 * @author fplanque: Francois PLANQUE.
 *
 * @package templates
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

load_funcs( 'templates/_template.funcs.php' );

// Check permission to display:
$current_User->check_perm( 'options', 'view', true );

param( 'action', 'string', 'list' );

param( 'redirect_to', 'string', '?ctrl=templates' );

if( param( 'template_ID', 'integer', '', true) )
{// Load file type:
	$TemplateCache = & get_Cache( 'TemplateCache' );
	if( ($edited_Template = & $TemplateCache->get_by_ID( $template_ID, false )) === false )
	{	// We could not find the template to edit:
		unset( $edited_Template );
		forget_param( 'template_ID' );
		$Messages->head = T_('Cannot edit template!');
		$Messages->add( T_('Requested template is not installed any longer.'), 'error' );
		$action = 'nil';
	}
}


/**
 * Perform action:
 */
switch( $action )
{
	case 'create':
		param( 'template_folder', 'string', true );
		// Check validity of requested template name:
		if( preg_match( '~([^-A-Za-z0-9._]|\.\.)~', $template_folder ) )
		{
			debug_die( 'The requested template name is invalid.' );
		}

		// Check permission to edit:
		$current_User->check_perm( 'options', 'edit', true );

		// CREATE NEW TEMPLATE:
		$edited_Template = & template_install( $template_folder );
		if( ! $Messages->count( 'error' ) )
		{
			$Messages->add( T_('Template has been installed.'), 'success' );

			// We want to highlight the edited object on next list display:
	 		$Session->set( 'fadeout_array', array( 'template_ID' => array($edited_Template->ID) ) );
		}
		else
		{
			$Messages->add( T_('Template could not be installed.'), 'error' );
		}


		// PREVENT RELOAD & Switch to list mode:
		header_redirect( $redirect_to );
		break;


	case 'update':
		// Update template properties:

		// Check permission:
		$current_User->check_perm( 'options', 'edit', true );

		// Make sure we got an template_ID:
		param( 'template_ID', 'integer', true );

		// load data from request
		if( $edited_Template->load_from_Request() )
		{	// We could load data from form without errors:
			// Update in DB:
			$edited_Template->dbupdate();
			$Messages->add( T_('Template properties updated.'), 'success' );

			// We want to highlight the edited object on next list display:
 			$Session->set( 'fadeout_array', array( 'template_ID' => array($edited_Template->ID) ) );

			$action = 'list';
		}
		break;


	case 'reload':
		// Reload containers:

 		// Check permission:
		$current_User->check_perm( 'options', 'edit', true );

		// Make sure we got an template_ID:
		param( 'template_ID', 'integer', true );

		// Look for containers in template file:
		$edited_Template->discover_containers();

		// Save to DB:
		$edited_Template->db_save_containers();

		// We want to highlight the edited object on next list display:
 		$Session->set( 'fadeout_array', array( 'template_ID' => array($edited_Template->ID) ) );

		$action = 'list';
		break;


	case 'delete':
		// Uninstall a template:

		// Check permission:
		$current_User->check_perm( 'options', 'edit', true );

		// Make sure we got an template_ID:
		param( 'template_ID', 'integer', true );

		if( param( 'confirm', 'integer', 0 ) )
		{ // confirmed, Delete from DB:
			$msg = sprintf( T_('Template &laquo;%s&raquo; uninstalled.'), $edited_Template->dget('name') );
			$edited_Template->dbdelete( true );
			unset( $edited_Template );
			forget_param( 'template_ID' );
			$Messages->add( $msg, 'success' );

			// PREVENT RELOAD & Switch to list mode:
			header_redirect( '?ctrl=templates' );
		}
		else
		{	// not confirmed, Check for restrictions:
			if( ! $edited_Template->check_delete( sprintf( T_('Cannot uninstall template &laquo;%s&raquo;'), $edited_Template->dget('name') ) ) )
			{	// There are restrictions:
				$action = 'edit';
			}
		}

		break;
}


$AdminUI->set_path( 'options', 'templates' );

// Display <html><head>...</head> section! (Note: should be done early if actions do not redirect)
$AdminUI->disp_html_head();

// Display title, menu, messages, etc. (Note: messages MUST be displayed AFTER the actions)
$AdminUI->disp_body_top();

// Begin payload block:
$AdminUI->disp_payload_begin();

/**
 * Display Payload:
 */
switch( $action )
{
	case 'new':
		// Display VIEW:
		$AdminUI->disp_view( 'templates/views/_template_list_available.view.php' );
		break;

	case 'delete':
		// We need to ask for confirmation:
		$edited_Template->confirm_delete(
				sprintf( T_('Uninstall template &laquo;%s&raquo;?'),  $edited_Template->dget( 'name' ) ),
				$action, get_memorized( 'action' ) );
	case 'edit':
	case 'update':	// we return in this state after a validation error
		// Display VIEW:
		$AdminUI->disp_view( 'templates/views/_template.form.php' );
		break;

	case 'list':
		// Display VIEW:
		$AdminUI->disp_view( 'templates/views/_template_list.view.php' );
		break;
}

// End payload block:
$AdminUI->disp_payload_end();

// Display body bottom, debug info and close </html>:
$AdminUI->disp_global_footer();

?>
