<?php
/**
 * This file implements the AdSense plugin for Quam Plures
 *
 * Quam Plures - {@link http://quamplures.net/}
 * Released under GNU GPL License - {@link http://quamplures.net/license.html}
 * @copyright (c) 2009 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 *
 * @todo (legacy): Remove it in XML Feeds?
 *
 * @package plugins
 * @subpackage AdSense
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );


/**
 * @package plugins
 * @subpackage AdSense
 */
class adsense_plugin extends Plugin
{
	var $apply_rendering = 'opt-out';
	var $author = 'Dracones Incompertus'; // this is default
	var $code = 'adsense';
	var $group = 'toolbars';
	var $sub_group; // only used when $group is 'other'
	var $help_url = '';
	var $number_of_installs = 1;
	var $priority = 10;
	var $version = '0.1';


	/**
	 * @see Plugin::PluginInit()
	 */
	function PluginInit( & $params )
	{
		$this->name = $this->T_('AdSense');
		$this->short_desc = $this->T_('Easy AdSense insertion into posts.');
		$this->long_desc = $this->T_('<p>This plugin allows you to easily insert AdSense into your posts (simply type [adsense:], or use the toolbar button in expert mode).</p>
<p>The plugin will stop expanding AdSense blocks when a limit is reached (3 by default).</p>
<p>Look for version 2.0 for multiple AdSense format support.</p>');
	}


	/**
	 * @see Plugin::GetDefaultSettings()
	 */
	function GetDefaultSettings( & $params )
	{
		$default_settings = array(
			'enable_user' => array(
				'label' => $this->T_('enable user option'),
				'type' => 'checkbox',
				'defaultvalue' => '1',
				'note' => $this->T_('If checked, users can turn off this toolbar.'),
			),
			'toolbar_location' => array(
				'label' => $this->T_('toolbar location'),
				'type' => 'select',
				'options' => array( 
					'above' => $this->T_('above post textarea'), 
					'below' => $this->T_('below post textarea'),
					'right' => $this->T_('top of the sidebar'),
				),
				'defaultvalue' => 'right',
				'note' => $this->T_('Users will not be able to select where this toolbar goes.'),
			),

			'adsense_block' => array(
					'label' => $this->T_('AdSense block'),
					'type' => 'html_textarea',
					'cols' => 60,
					'rows' => 10,
					'defaultvalue' => '<div style="float:right; clear:both; margin:5px;">

<!-- Paste from here... -->
<div style="border:1px solid red; width:150px; padding:3px;"><strong>You must copy/paste your AdSense code in here.</strong> You can do this on the Plugin config page.</div>
<!-- ...to here -->

</div>',
					'note' => $this->T_('Copy/Paste your AdSense code from Google into here. You can surround it with some CSS for decoration and/or positioning.'),
				),
			'max_blocks_in_content' => array(
					'label' => $this->T_('Max # of blocks'),
					'type' => 'integer',
					'size' => 2,
					'maxlength' => 2,
					'defaultvalue' => 3,
					'note' => $this->T_('Maximum number of AdSense blocks the plugin should expand in post contents. Google terms typically set the limit to 3. You may wish to set it to less if you add blocks into the sidebar.'),
				),
			);
		return $default_settings;
	}


	/**
	 * @see Plugin::GetDefaultUserSettings()
	 */
	function GetDefaultUserSettings( & $params )
	{
		if( ! $this->Settings->get( 'enable_user' ) )
		{
			return false;
		}
		$user_settings = array(
			'use_toolbar' => array(
				'label' => sprintf( $this->T_('"%s" toolbar'), $this->name ),
				'defaultvalue' => '1',
				'type' => 'checkbox',
				'note' => sprintf( $this->T_('Un-check this if you do not want to use the "%s" toolbar.'), $this->name ),
			),
		);
		return $user_settings;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarAbove()
	 */
	function AdminDisplayToolbarAbove( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'above') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarBelow()
	 */
	function AdminDisplayToolbarBelow( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'below') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarSidebar()
	 */
	function AdminDisplayToolbarSidebar( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'right') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


  /**
   * Comments out the adsense tags so that they don't get worked on by other renderers like Auto-P
   *
	 * @param mixed $params
	 */
	function FilterItemContents( & $params )
	{
		$content = & $params['content'];

		$content = preg_replace( '\[(adsense:)\]', '<!-- [$1] -->', $content );

		return true;
	}


	/**
	 * Changes the commented out tags into something that is visible to the editor
	 *
	 * @param mixed $params
	 */
	function UnfilterItemContents( & $params )
	{
		$content = & $params['content'];

		$content = preg_replace( '<!-- \[(adsense:)\] -->', '[$1]', $content );

		return true;
	}


	/**
	 * @see Plugin::RenderItemAsHtml()
	 */
	function RenderItemAsHtml( & $params )
	{
		// Dummy placeholder. Without it the plugin would ne be considered to be a renderer...
		return false;
	}


	/**
	 * @see Plugin::DisplayItemAsHtml()
	 */
	function DisplayItemAsHtml( & $params )
	{
		$content = & $params['data'];

		$content = preg_replace_callback( '<!-- \[adsense:\] -->', array( $this, 'DisplayItem_callback' ), $content );

		return true;
	}


  /**
	 * foo?
	 */
	function DisplayItem_callback( $matches )
	{
	  /**
		 * How many blocks already displayed?
		 */
		static $adsense_blocks_counter = 0;

		$adsense_blocks_counter++;

		if( $adsense_blocks_counter > $this->Settings->get( 'max_blocks_in_content' ) )
		{
			return '<!-- Adsense block #'.$adsense_blocks_counter.' not displayed since it exceed the limit of '
							.$this->Settings->get( 'max_blocks_in_content' ).' -->';
		}

		return $this->Settings->get( 'adsense_block' );
	}


	/**
	 * @see Plugin::DisplayItemAsXml()
	 */
	function DisplayItemAsXml( & $params )
	{
		$content = & $params['data'];

		$content = preg_replace( '\[adsense:\]', '', $content );

		return true;
	}


	/**
	 * actually finally create the toolbar :)
	 */
	function make_the_toolbar( $params )
	{
		echo '<div class="edit_toolbar">';
		echo '<input type="button" id="adsense_default" title="'.$this->T_('Insert AdSense block').'" class="quicktags" onclick="textarea_wrap_selection( app_Canvas, \'[adsense:]\', \'\', 1 );" value="'.$this->T_('AdSense').'" />';
		echo '</div>';

		return true;
	}

}
?>
