<?php
/**
 * This file implements the Comment list Widget class.
 *
 * This file is part of Quam Plures - {@link http://quamplures.net/}
 * See also {@link https://launchpad.net/quam-plures}.
 *
 * @copyright (c) 2009 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 *
 * {@internal License choice
 * - If you have received this file as part of a package, please find the license.txt file in
 *   the same folder or the closest folder above for complete license terms.
 * - If you have received this file individually (e-g: from http://evocms.cvs.sourceforge.net/)
 *   then you must choose one of the following licenses before using the file:
 *   - GNU General Public License 2 (GPL) - http://www.opensource.org/licenses/gpl-license.php
 *   - Mozilla Public License 1.1 (MPL) - http://www.opensource.org/licenses/mozilla1.1.php
 * }}
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file: }}
 * @author fplanque: Francois PLANQUE.
 * @author Yabba	- {@link http://www.astonishme.co.uk/}
 *
 * @package widgets
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

load_class( 'widgets/model/_widget.class.php' );


/**
 * @package widgets
 */
class comment_list_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function comment_list_Widget( $db_row = NULL )
	{
		// Call parent constructor:
		parent::ComponentWidget( $db_row, 'widget', 'comment_list' );

		$this->widget_name = $this->T_('Comment list');
		$this->widget_title = $this->T_('Recent comments');
	}


  /**
   * Get definitions for editable params
   *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'disp_order' => array(
				'label' => $this->T_('Order'),
				'note' => $this->T_('Order to display items'),
				'type' => 'select',
				'options' => array( 'DESC' => $this->T_('Newest to oldest'), 'ASC' => $this->T_('Oldest to newest'), 'RAND' => $this->T_('Random selection') ),
				'defaultvalue' => 'DESC',
			),
			'limit' => array(
				'label' => $this->T_('Max items'),
				'note' => $this->T_('Maximum number of items to display.'),
				'size' => 4,
				'defaultvalue' => 20,
			),
			'author_links' => array(
				'label' => $this->T_('Link to author'),
				'note' => $this->T_('Link the author to their url'),
				'defaultvalue' => true,
				'type' => 'checkbox',
			),
			'hover_text' => array(
				'label' => $this->T_('Hover text'),
				'note' => $this->T_('Text to show when hovering over the link'),
				'size' => 40,
				'defaultvalue' => $this->T_('Read the full comment'),
				'type' => 'text',
			),
			'blog_ID' => array(
				'label' => $this->T_('Blog'),
				'note' => $this->T_('ID of the blog to use, leave empty for the current blog.'),
				'size' => 4,
			),
		), parent::get_param_definitions( $params )	);

		return $r;
	}


	/**
	 * Get a very short desc. Used in the widget list.
	 */
	function get_short_desc()
	{
		return format_to_output($this->disp_params['widget_title']);
	}


  /**
	 * Get short description
	 */
	function get_desc()
	{
		return $this->T_('List of comments; click goes to comment.');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		$this->init_display( $params );

		global $Blog;

		$blogCache = get_Cache( 'BlogCache' );
		$listBlog = ( $this->disp_params[ 'blog_ID' ] ? $blogCache->get_by_ID( $this->disp_params[ 'blog_ID' ] ) : $Blog );

		// Create ItemList
		// Note: we pass a widget specific prefix in order to make sure to never interfere with the mainlist
		$limit = $this->disp_params[ 'limit' ];
		$order = $this->disp_params[ 'disp_order' ];

		$CommentList = new CommentList( $listBlog, "'comment','trackback','pingback'", array('published'), '',	'',	$order,	'',	$limit );

		echo $this->disp_params[ 'block_start'];

		// Display title if requested
		$this->disp_title();

		echo $this->disp_params[ 'list_start' ];

    /**
		 * @var Comment
		 */
		while( $Comment = & $CommentList->get_next() )
		{ // Loop through comments:
			// Load comment's Item object:
			$Comment->get_Item();
			echo $this->disp_params[ 'item_start' ];
			$Comment->author( array(
				'after_visitor' => ' ',
				'after_user' => ' ',
				'make_link' => $this->disp_params[ 'author_links' ],
			) );
			echo $this->T_('on').' ';
			$Comment->permanent_link( array(
				'text'  => $Comment->Item->title,
				'title' => $this->disp_params[ 'hover_text' ],
				) );
			echo $this->disp_params[ 'item_end' ];
		}	// End of comment loop.}
		echo $this->disp_params[ 'list_end' ];
		echo $this->disp_params[ 'block_end' ];

		return true;
	}
}


?>