<?php
/**
 * This file implements the Image Smilies Renderer plugin for Quam Plures
 *
 * Quam Plures - {@link http://quamplures.net/}
 * Released under GNU GPL License - {@link http://quamplures.net/license.html}
 * @copyright (c) 2009 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 *
 * @author fplanque: Francois PLANQUE.
 * @author gorgeb: Bertrand GORGE / EPISTEMA
 *
 * @package plugins
 * @subpackage Smilies
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );


/**
 * @package plugins
 * @subpackage Smilies
 */
class smilies_plugin extends Plugin
{
	var $apply_rendering = 'opt-in';
	var $author = 'Dracones Incompertus'; // this is default
	var $code = 'qp_emoticons';
	var $group = 'toolbars';
	var $sub_group; // only used when $group is 'other'
	var $help_url; // defaults to ... dunno yet :)
	var $number_of_installs = 1;
	var $priority = 15;
	var $version = '0.1';
	// internal use vars...
	var $search;
	var $replace;
	var $smilies;


	/**
	 * @see Plugin::PluginInit()
	 */
	function PluginInit( & $params )
	{
		$this->name = $this->T_('Smilies');
		$this->short_desc = $this->T_('Graphical smileys');
		$this->long_desc = $this->T_('This renderer will convert text smilies like :) to graphical icons.<br />
			Optionally, it will also display a toolbar for quick insertion of smilies into a post.');
	}


	/**
	 * @see Plugin::GetDefaultSettings()
	 */
	function GetDefaultSettings( & $params )
	{
		global $rsc_subdir;
		$default_settings = array(
			'enable_user' => array(
				'label' => $this->T_('enable user option'),
				'type' => 'checkbox',
				'defaultvalue' => '1',
				'note' => $this->T_('If checked, users can turn off this toolbar.'),
			),
			'toolbar_location' => array(
				'label' => $this->T_('toolbar location'),
				'type' => 'select',
				'options' => array(
					'above' => $this->T_('above post textarea'),
					'below' => $this->T_('below post textarea'),
					'right' => $this->T_('top of the sidebar'),
				),
				'defaultvalue' => 'right',
				'note' => $this->T_('Users will not be able to select where this toolbar goes.'),
			),
			'render_comments' => array(	// fp> Note: this is not a default in this version, it's an 'always' :]
				'label' => $this->T_('Render comments' ),
				'note' => $this->T_('Check to also render smilies in comments.'),
				'defaultvalue' => 0,
				'type' => 'checkbox',
			),
			// TODO (yabs) : Display these as images and individual inputs
			'smiley_list' => array(
				'label' => $this->T_('Smiley list'),
				'note' => sprintf( $this->T_('This is the list of smileys [one per line], in the format : char_sequence image_file // optional comment<br />To disable a smiley, just add one or more spaces to the start of its setting<br />You can add new smiley images by uploading the images to the %s folder.' ), '<span style="font-weight:bold">'.$rsc_subdir.'smilies/</span>'),
				'type' => 'html_textarea', // allows smilies with "<" in them
				'rows' => 10,
				'cols' => 60,
				'defaultvalue' => '
:)       smiley.png
:D       smiley-grin.png
:p       smiley-razz.png
B)       smiley-cool.png
;)       smiley-wink.png
:>       smiley-twist.png
:roll:   smiley-roll.png
:oops:   smiley-red.png
:|       smiley-neutral.png
:-/      smiley-confuse.png
:(       smiley-sad.png
>:(      smiley-mad.png
:\'(     smiley-cry.png
:>>      smiley-mr-green.png
:DD      smiley-lol.png
:o       smiley-surprise.png
8|       smiley-eek.png
>:-[     smiley-evil.png' ,
			),
		);
		return $default_settings;
	}


	/**
	 * @see Plugin::GetDefaultUserSettings()
	 */
	function GetDefaultUserSettings( & $params )
	{
		if( ! $this->Settings->get( 'enable_user' ) )
		{
			return false;
		}
		$user_settings = array(
			'use_toolbar' => array(
				'label' => sprintf( $this->T_('"%s" toolbar'), $this->name ),
				'defaultvalue' => '1',
				'type' => 'checkbox',
				'note' => sprintf( $this->T_('Un-check this if you do not want to use the "%s" toolbar.'), $this->name ),
			),
		);
		return $user_settings;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarAbove()
	 */
	function AdminDisplayToolbarAbove( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'above') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarBelow()
	 */
	function AdminDisplayToolbarBelow( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'below') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarSidebar()
	 */
	function AdminDisplayToolbarSidebar( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'right') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * @see Plugin::DisplayCommentToolbar()
	 */
	function DisplayCommentToolbar( & $params )
	{
		if( ($this->Settings->get('render_comments')) || (is_logged_in() && $this->UserSettings->get('use_toolbar')) )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * actually finally create the toolbar :)
	 */
	function make_the_toolbar( $params )
	{
		$this->InitSmilies(); // check smilies cached

		$grins = '';
		$smiled = array();
		foreach( $this->smilies as $smiley )
		{
			if (!in_array($smiley[ 'image' ], $smiled))
			{
				$smiled[] = $smiley[ 'image'];
				$smiley[ 'code' ] = str_replace(' ', '', $smiley[ 'code' ]);
				$grins .= '<img src="'.$smiley[ 'image' ].'" title="'.$smiley[ 'code' ].'" alt="'.$smiley[ 'code' ]
									.'" class="top" onclick="textarea_wrap_selection( app_Canvas, \''. str_replace("'","\'",$smiley[ 'code' ]). '\', \'\', 1 );" /> ';
			}
		}

		echo '<div class="edit_toolbar" id="smiley-toolbar">'.$grins.'</div>' ;

		return true;
	}


	/**
	 * Perform rendering
	 *
	 * @see Plugin::FilterCommentContent()
	 */
	function FilterCommentContent( & $params )
	{
		if( $this->Settings->get( 'render_comments' ) )
		{
			$this->RenderItemAsHtml( $params );
		}
	}



	/**
	 * Perform rendering
	 *
	 * @see Plugin::RenderItemAsHtml()
	 */
	function RenderItemAsHtml( & $params )
	{
		$this->InitSmilies(); // check smilies are already cached


		if( ! isset( $this->search ) )
		{ // We haven't prepared the smilies yet
			$this->search = array();

			$tmpsmilies = $this->smilies;
			usort($tmpsmilies, array(&$this, 'smiliescmp'));

			foreach( $tmpsmilies as $smiley )
			{
				$this->search[] = $smiley[ 'code' ];
				$smiley_masked = '';
				for ($i = 0; $i < strlen($smiley[ 'code' ] ); $i++ )
				{
					$smiley_masked .=  '&#'.ord(substr($smiley[ 'code' ], $i, 1)).';';
				}

				// We don't use getimagesize() here until we have a mean
				// to preprocess smilies. It takes up to much time when
				// processing them at display time.
				$this->replace[] = '<img src="'.$smiley[ 'image' ].'" alt="'.$smiley_masked.'" class="middle" />';
			}
		}


		// REPLACE:  But only in non-HTML blocks, totally excluding <CODE>..</CODE> and <PRE>..</PRE>

		$content = & $params['data'];

		// Lazy-check first, using stristr() (stripos() is only available since PHP5):
		if( stristr( $content, '<code' ) !== false || stristr( $content, '<pre' ) !== false )
		{ // Call ReplaceTagSafe() on everything outside <pre></pre> and <code></code>:
			$content = callback_on_non_matching_blocks( $content,
					'~<(code|pre)[^>]*>.*?</\1>~is',
					array( & $this, 'ReplaceTagSafe' ) );
		}
		else
		{ // No CODE or PRE blocks, replace on the whole thing
			$content = $this->ReplaceTagSafe($content);
		}

		return true;
	}


	/**
	 * This callback gets called once after every tags+text chunk
	 * @return string Text with replaced smilies
	 */
	function preg_insert_smilies_callback( $text )
	{
		return str_replace( $this->search, $this->replace, $text );
	}


	/**
	 * Replace smilies in non-HTML-tag portions of the text.
	 * @uses callback_on_non_matching_blocks()
	 */
	function ReplaceTagSafe($text)
	{
		return callback_on_non_matching_blocks( $text, '~<[^>]*>~', array(&$this, 'preg_insert_smilies_callback') );
	}


	/**
	 * sorts the smilies' array by length
	 * this is important if you want :)) to superseede :) for example
	 */
	function smiliescmp($a, $b)
	{
		if( ($diff = strlen( $b[ 'code' ] ) - strlen( $a[ 'code' ] ) ) == 0)
		{
			return strcmp( $a[ 'code' ], $b[ 'code' ] );
		}
		return $diff;
	}


	/**
	 * Initiates the smiley array if not already initiated
	 *
	 * Attempts to use template specific smileys where available
	 *	- view_admin/template/rsc/smilies/
	 *	- templates/template/smilies/
	 *
	 * Attempts to fallback to default smilies
	 *	- rsc/smilies/
	 *
	 * If no image file found the smiley is not added
	 *
	 * @return array of available smilies( code, image url )
	 */
	function InitSmilies()
	{
		if( isset( $this->smilies ) )
		{ // smilies are already cached
			return;
		}

		global $admin_template, $admintemplates_path, $admintemplates_url, $rsc_path, $rsc_url, $template, $templates_path, $templates_url;

		// set the template path/url and the default (rsc) path/url
		$currenttemplate_path = ( is_admin_page() ? $admintemplates_path.$admin_template.'/rsc' : $templates_path.$template ).'/smilies/';
		$currenttemplate_url = ( is_admin_page() ? $admintemplates_url.$admin_template.'/rsc' : $templates_url.$template ).'/smilies/';
		$default_path = $rsc_path.'smilies/';
		$default_url = $rsc_url.'smilies/';

		$template_has_smilies = is_dir( $currenttemplate_path ); // check if template has a /smilies/ folder

		$this->smilies = array();
		$temp_list = explode( "\n", str_replace( array( "\r", "\t" ), '', $this->Settings->get( 'smiley_list' ) ) );

		foreach( $temp_list as $temp_smiley )
		{
			$a_smiley = explode( '<->',	preg_replace_callback( '#^(\S.+?\s)(.+?)(\/\/.*?)*$#', array( $this, 'get_smiley' ),$temp_smiley ) );
			if( isset( $a_smiley[0] ) and isset( $a_smiley[1] ) )
			{
				// lets see if the file exists
				$temp_img = trim( $a_smiley[1] );
				if( $template_has_smilies && is_file( $currenttemplate_path.$temp_img ) )
				{
					$temp_url = $currenttemplate_url.$temp_img; // template has it's own smiley, use it
				}
				elseif ( is_file( $default_path.$temp_img ) )
				{
					$temp_url = $default_url.$temp_img; // no template image, but default smiley found so use it
				}
				else
				{
					$temp_url = ''; // no smiley image found, so don't add the smiley
				}

				if( $temp_url )
					$this->smilies[] = array( 'code' => trim( $a_smiley[0] ),'image' => $temp_url );
			}
		}
	}

	/**
	 * returns the relevant smiley parts (char_code, image_file)
	 */
	function get_smiley( $smiley_parts )
	{
		return ( ( isset( $smiley_parts[1] ) && isset( $smiley_parts[2] ) ) ? $smiley_parts[1].'<->'.$smiley_parts[2] : '' );
	}

}
?>
