<?php
/**
 * This file implements the User Tools Widget class.
 *
 * This file is part of Quam Plures - {@link http://quamplures.net/}
 * See also {@link https://launchpad.net/quam-plures}.
 *
 * @copyright (c) 2009 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2009 by Francois PLANQUE - {@link http://fplanque.net/}
 *
 * {@internal License choice
 * - If you have received this file as part of a package, please find the license.txt file in
 *   the same folder or the closest folder above for complete license terms.
 * - If you have received this file individually (e-g: from http://evocms.cvs.sourceforge.net/)
 *   then you must choose one of the following licenses before using the file:
 *   - GNU General Public License 2 (GPL) - http://www.opensource.org/licenses/gpl-license.php
 *   - Mozilla Public License 1.1 (MPL) - http://www.opensource.org/licenses/mozilla1.1.php
 * }}
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file: }}
 * @author fplanque: Francois PLANQUE.
 *
 * @package widgets
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

load_class( 'widgets/model/_widget.class.php' );


/**
 * @package widgets
 */
class user_tools_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function user_tools_Widget( $db_row = NULL )
	{
		// Call parent constructor:
		parent::ComponentWidget( $db_row, 'widget', 'user_tools' );

		$this->widget_name = $this->T_('User Tools');
		$this->widget_title = $this->T_('User tools');
	}


  /**
   * Get definitions for editable params
   *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'user_login_link' => array(
				'label' => $this->T_('Login link'),
				'size' => 40,
				'note' => $this->T_('Link text to display'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Login'),
			),
			'user_logout_link' => array(
				'label' => $this->T_('Logout link'),
				'size' => 40,
				'note' => $this->T_('Link text to display'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Logout'),
			),
			'user_profile_link' => array(
				'label' => $this->T_('Profile link'),
				'size' => 40,
				'note' => $this->T_('Link text to display'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Profile'),
			),
			'user_subs_link' => array(
				'label' => $this->T_('Subscriptions link'),
				'size' => 40,
				'note' => $this->T_('Link text to display'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Subscriptions'),
			),
			'user_admin_link' => array(
				'label' => $this->T_('Admin link'),
				'size' => 40,
				'note' => $this->T_('Link text to display'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Admin'),
			),
			'user_register_link' => array(
				'label' => $this->T_('Register link'),
				'size' => 40,
				'note' => $this->T_('Link text to display'),
				'type' => 'text',
				'defaultvalue' => $this->T_('Register'),
			),
		), parent::get_param_definitions( $params )	);

		return $r;
	}


	/**
	 * Get a very short desc. Used in the widget list.
	 */
	function get_short_desc()
	{
		return format_to_output($this->disp_params['widget_title']);
	}


  /**
	 * Get short description
	 */
	function get_desc()
	{
		return $this->T_('Display user tools: Log in, Admin, Profile, Subscriptions, Log out');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		$this->init_display( $params );

		// User tools:
		echo $this->disp_params['block_start'];

		$this->disp_title();

		echo $this->disp_params['list_start'];
		user_login_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params[ 'user_login_link' ] );
		user_register_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params[ 'user_register_link' ] );
		user_admin_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params[ 'user_admin_link' ] );
		user_profile_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params[ 'user_profile_link' ] );
		user_subs_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params[ 'user_subs_link' ] );
		user_logout_link( $this->disp_params['item_start'], $this->disp_params['item_end'], $this->disp_params[ 'user_logout_link' ] );
		echo $this->disp_params['list_end'];

		echo $this->disp_params['block_end'];
	}
}


?>