<?php
/**
 * This file implements the Video Plug plugin for Quam Plures
 *
 * Quam Plures - {@link http://quamplures.net/}
 * Released under GNU GPL License - {@link http://quamplures.net/license.html}
 * @copyright (c) 2009 by the Quam Plures developers - {@link http://quamplures.net/}
 *
 * @package plugins
 * @subpackage Video
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

// dependancy
require_once dirname(__FILE__).'/_video.class.php';


/**
 * @package plugins
 * @subpackage Video
 */
class video_plugin extends Plugin
{
	var $apply_rendering = 'opt-out';
	var $author = 'Dracones Incompertus'; // this is default
	var $code = 'video_embed';
	var $group = 'toolbars';
	var $sub_group; // only used when $group is 'other'
	var $help_url = '';
	var $number_of_installs = 1;
	var $priority = 65;
	var $version = '0.1';
	// internal use vars...
	var $_video_modules = array();


	/**
	 * @see Plugin::PluginInit()
	 */
	function PluginInit( & $params )
	{
		$this->name = $this->T_('Videos');
		$this->short_desc = $this->T_('Embed videos from many popular video sites.');
		$this->long_desc = $this->T_('Many different sources of videos, you paste in the embed code (and frequently many other source-supplied ways to share a video), and it puts it on your web scaled to your template.');

		$modules = dirname(__FILE__).'/video_modules/';

		if( ( $files = @scandir( $modules ) ) !== false )
		{
			foreach( $files as $file )
			{
				if( preg_match( '~^_(.+?)\.vp\.php$~', $file, $matches ) )
				{
					require_once $modules.$file;
					$mod_class = 'vp_'.$matches[1];

					if( ! class_exists( $mod_class ) )
					{
						$this->debug_log( '['.$modules.$file.'] Class "'.
											$mod_class.'" does not exist!' );
						continue;
					}

					$mod_name = $matches[1];
					$this->_video_modules[ $mod_name ] = new $mod_class();
				}
			}
		}
	}


	/**
	 * @see Plugin::GetDefaultSettings()
	 */
	function GetDefaultSettings( & $params )
	{
		$default_settings = array(
			'primary_start' => array(
				'label' => $this->T_('Primary Settings'),
				'layout' => 'begin_fieldset',
			),
			'enable_user' => array(
				'label' => $this->T_('enable user option'),
				'type' => 'checkbox',
				'defaultvalue' => '1',
				'note' => $this->T_('If checked, users can turn off this toolbar.'),
			),
			'toolbar_location' => array(
				'label' => $this->T_('toolbar location'),
				'type' => 'select',
				'options' => array(
					'above' => $this->T_('above post textarea'),
					'below' => $this->T_('below post textarea'),
					'right' => $this->T_('top of the sidebar'),
				),
				'defaultvalue' => 'below',
				'note' => T_('Users will not be able to select where this toolbar goes.'),
			),
			'width' => array(
				'label' => $this->T_('Video Width'),
				'type' => 'integer',
				'defaultvalue' => 425,
				'valid_range' => array( 'min' => 320, 'max' => 640 ),
				'note' => $this->T_('The width you want for embedded videos on your blog (between 320 and 640).'),
			),
			'primary_end' => array(
				'layout' => 'end_fieldset',
			),
		);

		if( !empty( $this->_video_modules ) )
		{
			$default_settings['secondary_start'] = array(
				'label' => $this->T_('Additional Settings'),
				'layout' => 'begin_fieldset',
			);

			$extras = array();
			foreach( $this->_video_modules as $module => $obj )
			{
				$obj->init( $this );
				$default_settings['show_'.$module ] = array(
					'label' => $obj->nice_name,
					'type' => 'checkbox',
					'defaultvalue' => $obj->def_value,
					'note' => $this->T_('Enable button for').' <a href="'.$obj->homepage.'" title="'.$this->T_('opens in a new tab/window' ).'" target="_blank">'.$obj->homepage.'</a>?',
				);
				if( $more = $obj->GetExtraSettings() )
				{
					$extras += $more;
				}
			}

		$default_settings['secondary_end'] = array(
				'layout' => 'end_fieldset',
			);
			if( !empty( $extras ) )
			{
				$default_settings['tertiary_start'] = array(
					'label' => $this->T_('Even More Settings'),
					'layout' => 'begin_fieldset',
				);

				$default_settings += $extras;

				$default_settings['tertiary_end'] = array(
					'layout' => 'end_fieldset',
				);
			}
		}
	return $default_settings;
	}


	/**
	 * @see Plugin::GetDefaultUserSettings()
	 */
	function GetDefaultUserSettings()
	{
		if( ! $this->Settings->get( 'enable_user' ) )
		{
			return false;
		}
		$user_settings = array(
			'use_toolbar' => array(
				'label' => sprintf( $this->T_('"%s" toolbar'), $this->name ),
				'defaultvalue' => '1',
				'type' => 'checkbox',
				'note' => sprintf( $this->T_('Un-check this if you do not want to use the "%s" toolbar.'), $this->name ),
			),
		);
		return $user_settings;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarAbove()
	 */
	function AdminDisplayToolbarAbove( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'above') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarBelow()
	 */
	function AdminDisplayToolbarBelow( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'below') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * @see Plugin::AdminDisplayToolbarSidebar()
	 */
	function AdminDisplayToolbarSidebar( & $params )
	{
		$show_toolbar = ( $this->Settings->get('enable_user') ? $this->UserSettings->get('use_toolbar') : true );
		if( ($this->Settings->get('toolbar_location') == 'right') && $show_toolbar )
		{
			return $this->make_the_toolbar( $params );
		}
		return false;
	}


	/**
	 * actually finally create the toolbar :)
	 */
	function make_the_toolbar( $params )
	{
		if( !empty( $this->_video_modules ) )
		{
			$js_sources = array();

			foreach( $this->_video_modules as $mod => $obj )
			{
				if( $this->Settings->get('show_'.$mod ) )
				{
					$js_func = 'source :function(){
'.$obj->GetJavascript().'
},';
					$js_sources[] = 'appVideo.addSource({id:"'.$mod.'", name:"'.$obj->nice_name.'", '.$js_func.' });';
				}
			}
			if( !empty( $js_sources ) )
			{
				echo '<div id="app_video" class="edit_toolbar">&nbsp;'.$this->get_help_link('$readme').'</div>
<script type="text/javascript">
//<![CDATA[
var appVideo = new function(){

	return {
		addSource:function( video_source ){
			var _vBut = jQuery( \'<input type="button" value="\'+video_source.id+\'" onclick="appVideo.addVideo( this )" />\' );
			_vBut.data( "video_source", video_source );
			_vBut.prependTo( "#app_video");
		},

		addVideo:function( which ){
			var _vsrc = jQuery( which ).data("video_source" );
			var _prompt = "'.$this->T_( 'Paste the embed code for %s here' ).'";
			var _error = "'.$this->T_( 'That is not a valid %s code' ).'"
			if( vid_code = prompt( _prompt.replace( /%s/, _vsrc.name ) ) )
			{
				if( vid_embed = _vsrc.source( vid_code ) )
				{
					textarea_wrap_selection( app_Canvas, vid_embed, "", 1 );
				}
				else
				{
					window.alert( _error.replace( /%s/, _vsrc.name ) );
				}
			}
		}
	}
}
'.implode("\n", array_reverse( $js_sources ) ).'
//]]>
</script>'."\n";
			}
		}
		return true;
	}


	/**
	 * @see Plugin::RenderItemAsHtml()
	 */
	function RenderItemAsHtml( & $params )
	{
		$width = $this->Settings->get('width');
		$content = & $params['data'];

		if( !empty( $this->_video_modules ) )
		{
			foreach( $this->_video_modules as $mod => $obj )
			{
				if( $this->Settings->get('show_'.$mod) )
				{
					$content = $obj->RenderContent( $content, $width );
				}
			}
		}
		return true;
	}


	function DisplayItemAsHtml( & $params )
	{
		return;
		$content = & $params['data'];
		switch( $params['format'] )
		{
			case 'entityencoded' :
				$this->RenderItemAsXml( $params );
				break;

			default :
				break;
		}
	}

	/**
	 * @see Plugin::RenderItemAsXml()
	 */
	function RenderItemAsXml( & $params )
	{
		/**
		 * @todo EdB: this still doesn't show up in my feed reader even though the
		 * object tag is visible in the feed. Check how the hooks work and see about
		 * replacing the object with a canned image that says "embedded video here"?
		 */
		$permalink = $params['Item']->get_permanent_url();
		$content = & $params['data'];
		$movie_reel_image = $this->get_plugin_url().'movie_reel.jpg';
		$content = preg_replace( '#\[(video|object):.+?]#', '<div class="videoblock"><a href="'.$permalink.'" title="'.T_('view embedded video by clicking through to my site').'"><img src="'.$movie_reel_image.'" width="240" height="212" alt="'.T_('view embedded video by clicking through to my site').'" /></a></div>', $content );
	}
}
?>
