<?php
/**
 * This file provides icon definitions through a function.
 *
 * Will resolve translations at runtime and consume less memory than a table.
 *
 * @package pond
 */
if( !defined('QP_CONFIG_LOADED') ) die( 'Please, do not access this page directly.' );


/**
 * Get icon according to an item.
 *
 * @param string icon name/key
 * @return array array( 'file' (relative to $rsc_path/$rsc_url), 'alt', 'size', 'class', 'rollover' )
 */
function get_icon_info($name)
{
	switch($name)
	{
		case 'pixel': return array( //
			'file' => 'icons/blank.gif',
			'alt'  => '',
			'size' => array( 1, 1 ),
		);

		case 'dropdown': return array( //
			'file' => 'icons/png/dropdown.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'switch-to-admin': return array( //
			'file' => 'icons/png/hammer-screwdriver.png',
			'alt'  => /* TRANS: short for "Switch to _A_dmin" */ T_('Adm'),
			'size' => array( 16, 16 ),
		);
		case 'switch-to-blog': return array( //
			'file' => 'icons/png/blog.png',
			'alt'  => /* TRANS: short for "Switch to _B_log" */ T_('Blg'),
			'size' => array( 16, 16 ),
		);

		case 'folder': return array( // 
			'file' => 'icons/png/document/folder.png',
			'alt'  => T_('Folder'),
			'size' => array( 16, 16 ),
		);
		case 'file_unknown': return array( // 
			'file' => 'icons/png/document/default.png',
			'alt'  => T_('Unknown file'),
			'size' => array( 16, 16 ),
		);
		case 'file_empty': return array( // 
			'file' => 'icons/png/document/empty.png',
			'alt'  => T_('Empty file'),
			'size' => array( 16, 16 ),
		);
		case 'folder_parent': return array( // 
			'file' => 'icons/png/up.png',
			'alt'  => T_('Parent folder'),
			'size' => array( 16, 16 ),
		);
		case 'folder_home': return array( // 
			'file' => 'icons/png/home_network.png',
			'alt'  => T_('Home folder'),
			'size' => array( 16, 16 ),
		);
		case 'file_edit': return array( // edit a file
			'file' => 'icons/png/document--pencil.png',
			'alt'  => T_('Edit'),
			'size' => array( 16, 16 ),
		);
		case 'file_copy': return array( // copy a file/folder
			'file' => 'icons/png/copy.png',
			'alt'  => T_('Copy'),
			'size' => array( 16, 16 ),
		);
		case 'file_move': return array( // move a file/folder
			'file' => 'icons/png/document--move.png',
			'alt'  => T_('Move'),
			'size' => array( 16, 16 ),
		);
		case 'file_rename': return array( // rename a file/folder
			'file' => 'icons/png/filerename.png',
			'alt'  => T_('Rename'),
			'size' => array( 16, 16 ),
		);
		case 'file_delete': return array( // delete a file/folder
			'file' => 'icons/png/document--cross.png',
			'alt'  => T_('Del'),
			'legend'=>T_('Delete'),
			'size' => array( 16, 16 ),
		);
		case 'file_perms': return array( // edit permissions of a file
			'file' => 'icons/png/fileperms.png',
			'alt'  => T_('Permissions'),
			'size' => array( 16, 16 ),
		);


		case 'ascending': return array( // ascending sort order
			'file' => 'icons/png/sort-alphabet-az.png',
			'alt'  => /* TRANS: Short (alt tag) for "Ascending" */ T_('A'),
			'size' => array( 16, 16 ),
		);
		case 'descending': return array( // descending sort order
			'file' => 'icons/png/sort-alphabet-za.png',
			'alt'  => /* TRANS: Short (alt tag) for "Descending" */ T_('D'),
			'size' => array( 16, 16 ),
		);

		case 'window_new': return array(    // open in a new window
			'file' => 'icons/png/window-new.png',
			'alt'  => T_('New window'),
			'size' => array( 16, 16 ),
		);


		case 'file_word': return array( //
			'ext'  => '\.(s[txd]w|doc|rtf)',
			'file' => 'icons/png/document/word.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_image': return array( //
			'ext'  => '\.(gif|png|jpe?g)',
			'file' => 'icons/png/document/image.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_www': return array(
			'ext'  => '\.html?',
			'file' => 'icons/png/document/globe.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_log': return array(
			'ext'  => '\.log',
			'file' => 'icons/png/document/log.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_sound': return array(
			'ext'  => '\.(mp3|ogg|wav)',
			'file' => 'icons/png/document/sound.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_video': return array(
			'ext'  => '\.(mpe?g|avi)',
			'file' => 'icons/png/document/film.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_message': return array(
			'ext'  => '\.msg',
			'file' => 'icons/png/document/outlook.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_document': return array(
			'ext'  => '\.pdf',
			'file' => 'icons/png/document/pdf.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_php': return array(
			'ext'  => '\.php[34]?',
			'file' => 'icons/png/document/code.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_encrypted': return array(
			'ext'  => '\.(pgp|gpg)',
			'file' => 'icons/png/document/encrypted.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_tar': return array(
			'ext'  => '\.tar',
			'file' => 'icons/png/document/tar.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_tgz': return array(
			'ext'  => '\.tgz',
			'file' => 'icons/png/document/tgz.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_document': return array(
			'ext'  => '\.te?xt',
			'file' => 'icons/png/document/document.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_pk': return array(
			'ext'  => '\.(zip|rar)',
			'file' => 'icons/png/document/tar.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);


		case 'expand': return array( //
			'file' => 'icons/png/expand.png',
			'alt'  => '+',
			'legend' => T_('Expand'),
			'size' => array( 16, 16 ),
		);
		case 'collapse': return array( //
			'file' => 'icons/png/collapse.png',
			'alt'  => '-',
			'legend' => T_('Collapse'),
			'size' => array( 16, 16 ),
		);

		case 'refresh': return array( //
			'file' => 'icons/png/refresh.png',
			'alt'  => T_('Refresh'),
			'size' => array( 16, 16 ),
		);
		case 'reload': return array( //
			'file' => 'icons/png/reload.png',
			'alt'  => T_('Reload'),
			'size' => array( 16, 16 ),
		);

		case 'download': return array( //
			'file' => 'icons/png/download-manager.png',
			'alt'  => T_('Download'),
			'size' => array( 16, 16 ),
		);


		case 'warning': return array( //
			'file' => 'icons/png/warning.png', 
			'alt'  => T_('Warning'),
			'size' => array( 16, 16 ),
		);

		case 'info': return array( //
			'file' => 'icons/png/info.png',
			'alt'  => T_('Info'),
			'size' => array( 16, 16 ),
		);
		case 'email': return array( //
			'file' => 'icons/png/mail.png',
			'alt'  => T_('Email'),
			'size' => array( 16, 16 ),
		);
		case 'www': return array(  //
			'file' => 'icons/png/url.png',
			'alt'  => T_('WWW'),
			'legend' => T_('Website'),
			'size' => array( 32, 16 ),
		);

		case 'new': return array( //
			'file' => 'icons/png/new.png',
			'alt'  => T_('New'),
			'size' => array( 16, 16 ),
		);
		case 'copy': return array( //
			'file' => 'icons/png/copy.png',
			'alt'  => T_('Copy'),
			'size' => array( 16, 16 ),
		);
		case 'edit': return array( //
			'file' => 'icons/png/document--pencil.png',
			'alt'  => T_('Edit'),
			'size' => array( 16, 16 ),
		);
		case 'properties': return array( //
			'file' => 'icons/png/properties.png',
			'alt'  => T_('Properties'),
			'size' => array( 16, 16 ),
		);
		case 'publish': return array(
			'file' => 'icons/png/blog--plus.png',
			'alt'  => T_('Publish'),
			'size' => array( 12, 16 ),
		);
		case 'deprecate': return array(
			'file' => 'icons/png/blog--minus.png',
			'alt'  => T_('Deprecate'),
			'size' => array( 12, 16 ),
		);
		case 'locate': return array( //
			'file' => 'icons/png/target.png',
			'alt'  => T_('Locate'),
			'size' => array( 16, 16 ),
		);
		case 'delete': return array( //
			'file' => 'icons/png/document--cross.png',
			'alt'  => T_('Del'),
			'legend' => T_('Delete'),
			'size' => array( 16, 16 ),
		);
		case 'close': return array( //
			'file' => 'icons/png/close.png',
			'rollover' => true,
			'alt' => T_('Close'),
			'size' => array( 16, 16 ),
		);


		case 'increase': return array( //
			'file' => 'icons/png/increase.png',
			'rollover' => true,
			'alt' => T_('+'),
			'size' => array( 16, 16 ),
		);
		case 'decrease': return array( //
			'file' => 'icons/png/decrease.png',
			'rollover' => true,
			'alt' => T_('-'),
			'size' => array( 16, 16 ),
		);

		case 'bullet_full': return array( //
			'file' => 'icons/png/bullet-full.png',
			'alt'  => '&bull;',
			'size' => array( 9, 9 ),
		);
		case 'bullet_empty': return array( //
			'file' => 'icons/png/bullet-empty.png',
			'alt'  => '&nbsp;',
			'size' => array( 9, 9 ),
		);
		case 'bullet_red': return array( //
			'file' => 'icons/png/bullet-red.png',
			'alt'  => '&nbsp;',
			'size' => array( 9, 9 ),
		);

		case 'activate': return array(
			'file' => 'icons/png/activate.png',
			'alt'  => /* TRANS: Short for "Activate(d)" */ T_('Act.'),
			'legend' => T_('Activate'),
			'size' => array( 16, 16 ),
		);
		case 'deactivate': return array(
			'file' => 'icons/png/deactivate.png',
			'alt'  => /* TRANS: Short for "Deactivate(d)" */ T_('Deact.'),
			'legend' => T_('Deactivate'),
			'size' => array( 16, 16 ),
		);
		case 'enabled': return array(
			'file' => 'icons/png/bullet-full.png',
			'alt'  => /* TRANS: Short for "Activate(d)" */ T_('Act.'),
			'legend' => T_('Activated'),
			'size' => array( 9, 9 ),
		);
		case 'disabled': return array(
			'file' => 'icons/png/bullet-empty.png',
			'alt'  => /* TRANS: Short for "Deactivate(d)" */ T_('Deact.'),
			'legend' => T_('Deactivated'),
			'size' => array( 9, 9 ),
		);

		case 'link': return array( //
			'file' => 'icons/png/chain-link.png',
			/* TRANS: Link + space => verb (not noun) */ 'alt' => T_('Link '),
			'size' => array( 16, 16 ),
		);
		case 'unlink': return array( //
			'file' => 'icons/png/chain-unlink.png',
			'alt'  => T_('Unlink'),
			'size' => array( 16, 16 ),
		);

		case 'calendar': return array( //
			'file' => 'icons/png/calendar.png',
			'alt'  => T_('Calendar'),
			'size' => array( 16, 16 ),
		);

		case 'parent_childto_arrow': return array( //
			'file' => 'icons/png/parent-childto-arrow.png',
			'alt'  => T_('+'),
			'size' => array( 16, 16 ),
		);

		case 'help': return array(
			'file' => 'icons/png/question.png',
			'alt'  => T_('Help'),
			'size' => array( 16, 16 ),
		);
		case 'manual': return array( //
			'file' => 'icons/png/manual.png',
			'rollover' => true,
			'alt'  => T_('Help'),
			'legend' => T_('Online Manual'),
			'size' => array( 16, 16 ),
		);
		case 'permalink': return array( //
			'file' => 'icons/png/bean.png',
			'rollover' => true,
			'alt'  => T_('Permalink'),
			'size' => array( 11, 11 ),
		);
		case 'history': return array( //
			'file' => 'icons/png/clock.png',
			'alt'  => T_('History'),
			'size' => array( 16, 16 ),
		);

		case 'file_allowed': return array( //
			'file' => 'icons/png/lock-unlock.png',
			'alt'  => T_( 'Allowed' ),
			'size' => array( 16, 16 ),
		);
		case 'file_not_allowed': return array( //
			'file' => 'icons/png/lock-locked.png',
			'alt'  => T_( 'Blocked' ),
			'size' => array( 16, 16 ),
		);

		case 'comments': return array( //
			'file' => 'icons/png/comments.png',
			'alt'  => T_('Comments'),
			'size' => array( 16, 16 ),
		);
		case 'nocomment': return array( //
			'file' => 'icons/png/nocomments.png',
			'alt'  => T_('Comments'),
			'size' => array( 16, 16 ),
		);

		case 'move_up': return array( //
			'file' => 'icons/png/moveup.png',
			'alt'  => T_( 'Up' ),
			'size' => array( 16, 16 ),
		);
		case 'move_down': return array( //
			'file' => 'icons/png/movedown.png',
			'alt'  => T_( 'Down'),
			'size' => array( 16, 16 ),
		);
		case 'nomove_up': return array( //
			'file' => 'icons/png/nomoveup.png',
			'alt'  => T_( 'Sort by order' ),
			'size' => array( 16, 16 ),
		);
		case 'nomove_down': return array( //
			'file' => 'icons/png/nomovedown.png',
			'alt'  => T_( 'Sort by order' ),
			'size' => array( 16, 16 ),
		);
		case 'nomove': return array( //
			'file' => 'icons/png/nomove.png',
			'size' => array( 16, 16 ),
		);

		case 'assign': return array( //
			'file' => 'icons/png/hand-share.png',
			'alt'  => T_('Assigned to'),
			'size' => array( 16, 16 ),
		);
		case 'check_all': return array( //
			'file' => 'icons/png/check-all.png',
			'alt'  => T_('Check all'),
			'size' => array( 16, 16 ),
		);
		case 'uncheck_all': return array( //
			'file' => 'icons/png/uncheck-all.png',
			'alt'  => T_('Uncheck all'),
			'size' => array( 16, 16 ),
		);
 
		case 'reset_filters': return array( //
			'file' => 'icons/png/funnel-reset.png',
			'alt'  => T_('Reset all filters'),
			'size' => array( 16, 16 ),
		);

		case 'allowback': return array( //
			'file' => 'icons/png/tick.png',
			'alt'	 => T_('Allow back'),
			'size' => array( 16, 16 ),
		);
		case 'ban': return array( //
			'file' => 'icons/png/slash.png', 
			'alt'  => /* TRANS: Abbrev. */ T_('Ban'),
			'size' => array( 16, 16 ),
		);
		case 'play': return array( // 
			'file' => 'icons/png/control.png',
			'alt'  => '&gt;',
			'size' => array( 16, 16 ),
		);

		case 'feed': return array( //
			'file' => 'icons/png/feed.png',
			'alt'	 => T_('XML Feed'),
			'size' => array( 16, 16 ),
		);

		case 'star_on': return array(
			'file' => 'icons/png/heart-small-full.png',
			'alt'	 => '*',
			'size' => array( 12, 12 ),
		);
		case 'star_half': return array(
			'file' => 'icons/png/heart-small-half.png',
			'alt'	 => '+',
			'size' => array( 12, 12 ),
		);
		case 'star_off': return array(
			'file' => 'icons/png/heart-small-empty.png',
			'alt'	 => '-',
			'size' => array( 12, 12 ),
		);

		case 'file_css': return array(
			'ext'  => '\.css',
			'file' => 'icons/fileicons/css.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_mp3': return array(
			'ext'  => '\.(mp3)',
			'file' => 'icons/fileicons/mp3.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_m4a': return array(
			'ext'  => '\.(m4a)',
			'file' => 'icons/fileicons/m4a.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_mp4': return array(
			'ext'  => '\.(mp4)',
			'file' => 'icons/fileicons/mp4.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_mov': return array(
			'ext'  => '\.(mov)',
			'file' => 'icons/fileicons/mov.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_avi': return array(
			'ext'  => '\.(avi)',
			'file' => 'icons/png/document/avi.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_pls': return array(
			'ext'  => '\.(pls)',
			'file' => 'icons/fileicons/pls.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_video': return array(
			'ext'  => '\.(mpe?g)',
			'file' => 'icons/fileicons/video.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);
		case 'file_wmv': return array(
			'ext'  => '\.(wmv)',
			'file' => 'icons/fileicons/wmv.png',
			'alt'  => '',
			'size' => array( 16, 16 ),
		);

	}
}

?>
