<?php
/**
 * "Basic" template class
 *
 * This file is part of the Quam Plures project - {@link http://quamplures.net/}
 *
 * @package templates
 * @subpackage basic
 */
if( !defined('QP_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

/**
 * basic_Template class
 *
 * @package templates
 * @subpackage basic
 */
class basic_Template extends Template
{

	/**
	 * Get default name for the template, which the admin can customize.
	 */
	function get_default_name()
	{
		return 'Basic';
	}


	/**
	 * Get default type for the template.
	 */
	function get_default_type()
	{
		return 'normal';
	}


	/**
	 * Get definitions for editable params
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
				'style_sheet' => array(
					'label' => $this->T_('Style'),
					'type' => 'select',
					'note' => $this->T_('add some styles to the basic template ... which you probably won\'t like'),
					'options' => array(
						'none' => $this->T_('none'),
						'borders' => $this->T_('add borders'),
						'colors' => $this->T_('add colors'),
						'everything' => $this->T_('add borders and colors'),
						'outlines' => $this->T_('really cool outline thing'),
					),
					'defaultvalue' => 'none',
				),
			), parent::get_param_definitions( $params )	);
		return $r;
	}


	/**
	 * Get ready for displaying the template.
	 *
	 * This may register some CSS or JS...
	 */
	function display_init()
	{
		// call parent:
		parent::display_init();

		switch( $this->get_setting( 'style_sheet') )
		{
			case 'borders':
			$custom_css = '<link rel="stylesheet" type="text/css" href="style_borders.css" media="screen" />'."\n";
			break;

			case 'colors':
			$custom_css = '<link rel="stylesheet" type="text/css" href="style_colors.css" media="screen" />'."\n";
			break;

			case 'everything':
			$custom_css = '<link rel="stylesheet" type="text/css" href="style_everything.css" media="screen" />'."\n";
			break;

			case 'outlines':
			$custom_css = '<link rel="stylesheet" type="text/css" href="style_outlines.css" media="screen" />'."\n";
			break;

			default:
			$custom_css = '';
			break;
		}

		if( !empty( $custom_css ) )
		{
			add_headline( $custom_css );
		}

	}


	/**
	 * Get credits for linkback on credits page.
	 * see http://docs.quamplures.net/templates/Template.html#methodget_template_credits
	 */
	function get_template_credits()
	{
		$template_linkbacks = array(
			'inspired' => array(
				'pre' => T_('Inspired by'),
				'text' => 'b2evolution',
				'post' => '"basic" in b2evolution was for obsolete browsers. Here in QP it shows you almost every T-tag without any params, to help you design your own template.',
				'url' => 'http://b2evolution.net/',
				'title' => 'http://b2evolution.net/',
			),
			'ported' => array(
				'pre' => T_('Ported by'),
				'text' => 'EdB',
				'post' => T_(''),
				'url' => 'http://1912webworks.com/',
				'title' => T_('insert creative link title'),
			),
		);
		return $template_linkbacks;
	}

}

?>
